@@ -52,6 +52,8 @@
 #define O_BINARY 0
 #endif
 
+#define MAX_DICTS 128
+
 #define ASCII_LOW_T 0x21
 #define ASCII_HIGH_T 0x7E
 #define ASCII_VOTE_STRENGTH_T 150
@@ -71,11 +73,12 @@ extern int shammx_data( uchar ctx[40], uchar data[128], uchar buf[640] )
 __attribute__((regparm(3)));
 #endif
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev);
 extern int getmac(char * macAddress, int strict, unsigned char * mac);
 
 
 #define BROADCAST "\xFF\xFF\xFF\xFF\xFF\xFF"
+#define SPANTREE  "\x01\x80\xC2\x00\x00\x00"
 
 static uchar ZERO[32] =
 "\x00\x00\x00\x00\x00\x00\x00\x00"
@@ -111,8 +114,11 @@ struct options
 									multithreaded for SMP*/
 	int do_testy;				 /* experimental attack  */
 
-	FILE *dict;					 /* dictionary file      */
+	char *dicts[MAX_DICTS];			 /* dictionary files     */
+	FILE *dict;				 /* dictionary file      */
+	int nbdict;				 /* current dict number  */
 	int no_stdin;				 /* if dict == stdin     */
+	int hexdict[MAX_DICTS];			 /* if dict in hex       */
 
 	int showASCII;				 /* Show ASCII version of*/
 								 /* the wepkey           */
@@ -290,9 +296,11 @@ char usage[] =
 "      -y         : experimental single bruteforce mode\n"
 "      -s         : show ASCII version of the key\n"
 "\n"
-"  WPA-PSK cracking options:\n"
+"  WEP and WPA-PSK cracking options:\n"
 "\n"
-"      -w <words> : path to a dictionary file\n"
+"      -w <words> : path to a dictionary file (multiple\n"
+"                    dictionnaries can be specified.\n"
+"                    See manpage for more information)\n"
 /*"      -r <table> : path to a WPA PMK table\n" */
 "\n";
 
@@ -782,6 +790,8 @@ void read_thread( void *arg )
 		/* check minimum size */
 
 		z = ( ( h80211[1] & 3 ) != 3 ) ? 24 : 30;
+		if ( ( h80211[0] & 0x80 ) == 0x80 )
+			z+=2; /* 802.11e QoS */
 
 		if( z + 16 > (int) pkh.caplen )
 			goto unlock_mx_apl;
@@ -808,6 +818,14 @@ void read_thread( void *arg )
 
 			memcpy( buffer    , h80211 + z    , 3 );
 			memcpy( buffer + 3, h80211 + z + 4, 2 );
+			
+            /* Special handling for spanning-tree packets */
+            if ( memcmp( h80211 +  4, SPANTREE, 6 ) == 0 ||
+                memcmp( h80211 + 16, SPANTREE, 6 ) == 0 )
+            {
+                buffer[3] = (buffer[3] ^ 0x42) ^ 0xAA;
+                buffer[4] = (buffer[4] ^ 0x42) ^ 0xAA;
+            }
 
 			goto add_wep_iv;
 		}
@@ -848,8 +866,11 @@ void read_thread( void *arg )
 		ap_cur->crypt = 3;		 /* set WPA */
 
 		if( st_cur == NULL )
+		{
+			pthread_mutex_unlock( &mx_apl );
 			continue;
-
+		}
+		
 		/* frame 1: Pairwise == 1, Install == 0, Ack == 1, MIC == 0 */
 
 		if( ( h80211[z + 6] & 0x08 ) != 0 &&
@@ -859,7 +880,7 @@ void read_thread( void *arg )
 		{
 			memcpy( st_cur->wpa.anonce, &h80211[z + 17], 32 );
 
-								 /* authenticator nonce set */
+			/* authenticator nonce set */
 			st_cur->wpa.state = 1;
 		}
 
@@ -1316,7 +1337,7 @@ int check_wep_key( uchar *wepkey, int B, int keylen )
 {
 	uchar x1, x2;
 	unsigned long xv;
-	int i, j, n, bad;
+	int i, j, n, bad, tests;
 	int nb_ascii;
 
 	uchar K[64];
@@ -1330,7 +1351,14 @@ int check_wep_key( uchar *wepkey, int B, int keylen )
 
 	memcpy( K + 3, wepkey, keylen );
 
-	for( n = 0; n < 32; n++ )
+	tests = 32;
+
+	if(opt.dict) tests = (wep.nb_ivs-5)/5;
+
+	if(tests < 4)  tests=4;
+	if(tests > 32) tests=32;
+
+	for( n = 0; n < tests; n++ )
 	{
 		/* xv = 5 * ( rand() % wep.nb_ivs ); */
 		xv = 5 * n;
@@ -2237,6 +2265,61 @@ uchar mic[16], int force )
 	printf( "\n" );
 }
 
+int next_dict(int nb)
+{
+	if(opt.dict != NULL)
+	{
+		fclose(opt.dict);
+		opt.dict = NULL;
+	}
+	opt.nbdict = nb;
+	if(opt.dicts[opt.nbdict] == NULL)
+		return( FAILURE );
+
+	while(opt.nbdict < MAX_DICTS && opt.dicts[opt.nbdict] != NULL)
+	{
+		if( strcmp( opt.dicts[opt.nbdict], "-" ) == 0 )
+		{
+			if( ( opt.dict = fdopen( 0, "r" ) ) == NULL )
+			{
+				perror( "fopen(dictionary) failed" );
+				opt.nbdict++;
+				continue;
+			}
+		
+			opt.no_stdin = 1;
+		}
+		else
+		{
+			if( ( opt.dict = fopen( opt.dicts[opt.nbdict], "r" ) ) == NULL )
+			{
+				perror( "fopen(dictionary) failed" );
+				opt.nbdict++;
+				continue;
+			}
+		
+			fseek(opt.dict, 0L, SEEK_END);
+		
+			if ( ftell( opt.dict ) <= 0L )
+			{
+				fclose( opt.dict );
+				opt.dict = NULL;
+				printf( "Empty dictionnary\n" );
+				opt.nbdict++;
+				continue;
+			}
+		
+			rewind( opt.dict );
+		}
+		break;
+	}
+
+	if(opt.nbdict >= MAX_DICTS || opt.dicts[opt.nbdict] == NULL)
+	    return( FAILURE );
+
+	return( 0 );
+}
+
 int do_wpa_crack( struct AP_info *ap )
 {
 	int i, cid;
@@ -2247,6 +2330,8 @@ int do_wpa_crack( struct AP_info *ap )
 	uchar pmk2[40], ptk2[80];
 	uchar mic1[20], mic2[20];
 
+        i=0;
+
 	/* send the ESSID to each thread */
 
 	for( cid = 0; cid < opt.nbcpu; cid++ )
@@ -2307,6 +2392,12 @@ int do_wpa_crack( struct AP_info *ap )
 		{
 			/* read a couple of keys (skip those < 8 chars) */
 
+			if(opt.dict == NULL)
+			{
+				printf( "\nPassphrase not in dictionnary \n" );
+				return( FAILURE );
+			}
+
 			do
 			{
 				if( fgets( key1, sizeof( key1 ), opt.dict ) == NULL )
@@ -2314,9 +2405,15 @@ int do_wpa_crack( struct AP_info *ap )
 					if( opt.l33t )
 						printf( "\33[32;22m" );
 
-					printf( "\nPassphrase not in dictionnary\n" );
-
-					return( FAILURE );
+					printf( "\nPassphrase not in dictionnary %s \n", opt.dicts[opt.nbdict] );
+					if(next_dict(opt.nbdict+1) != 0)
+					{
+						return( FAILURE );
+					}
+					else
+					{
+						continue;
+					}
 				}
 
 				i = strlen( key1 );
@@ -2330,7 +2427,16 @@ int do_wpa_crack( struct AP_info *ap )
 			do
 			{
 				if( fgets( key2, sizeof( key2 ), opt.dict ) == NULL )
-					break;
+				{
+					if(next_dict(opt.nbdict+1) != 0)
+					{
+						break;
+					}
+					else
+					{
+						continue;
+					}
+				}
 
 				i = strlen( key2 );
 
@@ -2450,6 +2556,218 @@ void sighandler( int signum )
 		printf( "\33[2J\n" );
 }
 
+int next_key( char **key, int keysize )
+{
+	char *tmp, *tmp2;
+	int i, rtn;
+	unsigned int dec;
+	char *hex;
+
+	tmp2 = tmp = (char*) malloc(1024);
+
+	while(1)
+	{
+		rtn = 0;
+		tmp = tmp2;
+		if(opt.dict == NULL)
+		{
+			printf( "\nPassphrase not in dictionnary \n" );
+			free(tmp);
+			return( FAILURE );
+		}
+
+		if( opt.hexdict[opt.nbdict] )
+		{
+			if( fgets( tmp, ((keysize*2)+(keysize-1)), opt.dict ) == NULL )
+			{
+				if( opt.l33t )
+					printf( "\33[32;22m" );
+		
+//				printf( "\nPassphrase not in dictionnary \"%s\" \n", opt.dicts[opt.nbdict] );
+				if(next_dict(opt.nbdict+1) != 0)
+				{
+					free(tmp);
+					return( FAILURE );
+				}
+				else
+				{
+					continue;
+				}
+			}
+
+			i=strlen(tmp);
+
+			if( i <= 2 ) continue;
+
+			if( tmp[i - 1] == '\n' ) tmp[--i] = '\0';
+			if( tmp[i - 1] == '\r' ) tmp[--i] = '\0';
+			if( i <= 0 ) continue;
+
+			i=0;
+
+			hex = strsep(&tmp, ":");
+
+			while( i<keysize && hex != NULL )
+			{
+				if(strlen(hex) > 2 || strlen(hex) == 0)
+				{
+					rtn = 1;
+					break;
+				}
+				if(sscanf(hex, "%x", &dec) == 0 )
+				{
+					rtn = 1;
+					break;
+				}
+
+				(*key)[i] = dec;
+				hex = strsep(&tmp, ":");
+				i++;
+			}
+			if(rtn)
+			{
+				continue;
+			}
+		}
+		else
+		{
+			if( fgets( *key, keysize, opt.dict ) == NULL )
+			{
+				if( opt.l33t )
+					printf( "\33[32;22m" );
+		
+//				printf( "\nPassphrase not in dictionnary \"%s\" \n", opt.dicts[opt.nbdict] );
+				if(next_dict(opt.nbdict+1) != 0)
+				{
+					free(tmp);
+					return( FAILURE );
+				}
+				else
+				{
+					continue;
+				}
+			}
+
+			i=strlen(*key);
+
+			if( i <= 2 ) continue;
+
+			if( (*key)[i - 1] == '\n' ) (*key)[--i] = '\0';
+			if( (*key)[i - 1] == '\r' ) (*key)[--i] = '\0';
+
+			if( i <= 0 ) continue;
+		}
+
+		break;
+	}
+
+	free(tmp);
+
+	return( SUCCESS );
+}
+
+int set_dicts(char* optargs)
+{
+	int len;
+	char *optarg;
+
+	opt.nbdict = 0;
+	optarg = strsep(&optargs, ",");
+
+	for(len=0; len<MAX_DICTS; len++)
+	{
+		opt.dicts[len] = NULL;
+	}
+
+	while(optarg != NULL && opt.nbdict<MAX_DICTS)
+	{
+		len = strlen(optarg)+1;
+		opt.dicts[opt.nbdict] = (char*)malloc(len * sizeof(char));
+		if(opt.dicts[opt.nbdict] == NULL)
+		{
+			perror("allocation failed!");
+			return( FAILURE );
+		}
+		if(strncasecmp(optarg, "h:", 2) == 0)
+		{
+			strncpy(opt.dicts[opt.nbdict], optarg+2, len-2);
+			opt.hexdict[opt.nbdict] = 1;
+		}
+		else
+		{
+			strncpy(opt.dicts[opt.nbdict], optarg, len);
+			opt.hexdict[opt.nbdict] = 0;
+		}
+		optarg = strsep(&optargs, ",");
+		opt.nbdict++;
+	}
+
+	next_dict(0);
+
+	while(next_dict(opt.nbdict+1) == 0) {}
+
+	next_dict(0);
+
+	return 0;
+}
+
+int crack_wep_dict()
+{
+	struct timeval t_last;
+	struct timeval t_now;
+	int i, origlen, keysize;
+	char *key;
+
+	key = (char*) malloc(sizeof(char) * (opt.keylen + 1));
+	keysize = opt.keylen+1;
+
+	update_ivbuf();
+
+	if(wep.nb_ivs < 25)
+	{
+		printf( "\ncapture 25 IVs\n" );
+		return( FAILURE );
+	}
+
+	gettimeofday( &t_last, NULL );
+	t_last.tv_sec--;
+
+	while(1)
+	{
+		if( next_key( &key, keysize ) != SUCCESS) return( FAILURE );
+
+		i = strlen( key );
+
+		origlen = i;
+
+		while(i<opt.keylen)
+		{
+			key[i] = key[i - origlen];
+			i++;
+		}
+
+		key[i] = '\0';
+
+		if( ! opt.is_quiet )
+		{
+			gettimeofday( &t_now, NULL );
+			if( (t_now.tv_sec - t_last.tv_sec) > 0)
+			{
+				show_wep_stats(opt.keylen - 1, 1);
+				gettimeofday( &t_last, NULL);
+			}
+		}
+
+		for(i=0; i<=opt.keylen; i++)
+		{
+			wep.key[i] = (uchar)key[i];
+		}
+
+		if(check_wep_key(wep.key, opt.keylen, 0) == SUCCESS)
+			return( SUCCESS );
+	}
+}
+
 int main( int argc, char *argv[] )
 {
 	int i, n, ret, max_cpu, option;
@@ -2458,7 +2776,7 @@ int main( int argc, char *argv[] )
 
 	ret = FAILURE;
 
-	progname = getVersion("Aircrack-ng", _MAJ, _MIN, _SUB_MIN, _BETA);
+	progname = getVersion("Aircrack-ng", _MAJ, _MIN, _SUB_MIN, _REVISION);
 
 	memset( &opt, 0, sizeof( opt ) );
 
@@ -2697,35 +3015,8 @@ int main( int argc, char *argv[] )
 				break;
 
 			case 'w' :
-				if( strcmp( optarg, "-" ) == 0 )
-				{
-					if( ( opt.dict = fdopen( 0, "r" ) ) == NULL )
-					{
-						perror( "fopen(dictionary) failed" );
-						return( FAILURE );
-					}
-
-					opt.no_stdin = 1;
-				}
-				else
-				{
-					if( ( opt.dict = fopen( optarg, "r" ) ) == NULL )
-					{
-						perror( "fopen(dictionary) failed" );
-						return( FAILURE );
-					}
-
-					fseek(opt.dict, 0L, SEEK_END);
-
-					if ( ftell( opt.dict ) <= 0L )
-					{
-						fclose( opt.dict );
-						printf( "Empty dictionnary\n" );
-						return( FAILURE );
-					}
-
-					rewind( opt.dict );
-				}
+				if(set_dicts(optarg) != 0)
+				    return FAILURE;
 				break;
 
 			case '0' :
@@ -2993,74 +3284,81 @@ int main( int argc, char *argv[] )
 
 		memset( &wep, 0, sizeof( wep ) );
 
-		for( i = 0; i < opt.nbcpu; i++ )
+		if(opt.dict != NULL)
 		{
-			/* start one thread per cpu */
-
-			pthread_t tid;
-
-			if (opt.amode<=1 && opt.nbcpu>1 && opt.do_brute && opt.do_mt_brute)
+			crack_wep_dict();
+		}
+		else
+		{
+			for( i = 0; i < opt.nbcpu; i++ )
 			{
-				if (pthread_create( &tid, NULL, (void *) inner_bruteforcer_thread,
-					(void *) (long) i ) != 0)
+				/* start one thread per cpu */
+	
+				pthread_t tid;
+	
+				if (opt.amode<=1 && opt.nbcpu>1 && opt.do_brute && opt.do_mt_brute)
+				{
+					if (pthread_create( &tid, NULL, (void *) inner_bruteforcer_thread,
+						(void *) (long) i ) != 0)
+					{
+						perror( "pthread_create failed" );
+						goto exit_main;
+					}
+				}
+	
+				if( pthread_create( &tid, NULL, (void *) crack_wep_thread,
+					(void *) (long) i ) != 0 )
 				{
 					perror( "pthread_create failed" );
 					goto exit_main;
 				}
 			}
-
-			if( pthread_create( &tid, NULL, (void *) crack_wep_thread,
-				(void *) (long) i ) != 0 )
+	
+			if( ! opt.do_testy )
 			{
-				perror( "pthread_create failed" );
-				goto exit_main;
-			}
-		}
-
-		if( ! opt.do_testy )
-		{
-			do   { ret = do_wep_crack1( 0 ); }
-			while( ret == RESTART );
-
-			if( ret == FAILURE )
-			{
-				printf( "   Attack failed. Possible reasons:\n\n"
-					"     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
-					"       can be cracked with about one million IVs, sometimes more.\n\n"
-					"     * If all votes seem equal, or if there are many negative votes,\n"
-					"       then the capture file is corrupted, or the key is not static.\n\n"
-					"     * A false positive prevented the key from being found.  Try to\n"
-					"       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
-					"       (-f)" );
-				if (opt.do_testy)
-					printf( "and try the experimental bruteforce attacks (-y)." );
-				printf( "\n" );
-			}
-		}
-		else
-		{
-			for( i = opt.keylen - 3; i < opt.keylen - 2; i++ )
-			{
-				do   { ret = do_wep_crack2( i ); }
+				do   { ret = do_wep_crack1( 0 ); }
 				while( ret == RESTART );
-
-				if( ret == SUCCESS )
-					break;
+	
+				if( ret == FAILURE )
+				{
+					printf( "   Attack failed. Possible reasons:\n\n"
+						"     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
+						"       can be cracked with about one million IVs, sometimes more.\n\n"
+						"     * If all votes seem equal, or if there are many negative votes,\n"
+						"       then the capture file is corrupted, or the key is not static.\n\n"
+						"     * A false positive prevented the key from being found.  Try to\n"
+						"       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
+						"       (-f)" );
+					if (opt.do_testy)
+						printf( "and try the experimental bruteforce attacks (-y)." );
+					printf( "\n" );
+				}
 			}
-
-			if( ret == FAILURE )
+			else
 			{
-				printf( "   Attack failed. Possible reasons:\n\n"
-					"     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
-					"       can be cracked with about one million IVs, sometimes more.\n\n"
-					"     * If all votes seem equal, or if there are many negative votes,\n"
-					"       then the capture file is corrupted, or the key is not static.\n\n"
-					"     * A false positive prevented the key from being found.  Try to\n"
-					"       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
-					"       (-f)" );
-				if (opt.do_testy)
-					printf( "or try the standard attack mode instead (no -y option)." );
-				printf( "\n" );
+				for( i = opt.keylen - 3; i < opt.keylen - 2; i++ )
+				{
+					do   { ret = do_wep_crack2( i ); }
+					while( ret == RESTART );
+	
+					if( ret == SUCCESS )
+						break;
+				}
+	
+				if( ret == FAILURE )
+				{
+					printf( "   Attack failed. Possible reasons:\n\n"
+						"     * Out of luck: you must capture more IVs. Usually, 104-bit WEP\n"
+						"       can be cracked with about one million IVs, sometimes more.\n\n"
+						"     * If all votes seem equal, or if there are many negative votes,\n"
+						"       then the capture file is corrupted, or the key is not static.\n\n"
+						"     * A false positive prevented the key from being found.  Try to\n"
+						"       disable each korek attack (-k 1 .. 17), raise the fudge factor\n"
+						"       (-f)" );
+					if (opt.do_testy)
+						printf( "or try the standard attack mode instead (no -y option)." );
+					printf( "\n" );
+				}
 			}
 		}
 	}
