@@ -52,7 +52,7 @@
     "\xC1\x16\x02\x01\x08\x00\x83\xDC\x74\x22\x00\x01"
 
 
-extern char * getVersion(char * progname, int maj, int min, int submin, int betavers);
+extern char * getVersion(char * progname, int maj, int min, int submin, int svnrev);
 extern int getmac(char * macAddress, int strict, unsigned char * mac);
 extern int add_crc32(unsigned char* data, int length);
 
@@ -74,8 +74,9 @@ char usage[] =
 "      -j             : set FromDS bit\n"
 "      -o             : clear ToDS bit\n"
 "      -e             : disables WEP encryption\n"
-"      -k <ip[:port]> : set Source      IP [Port]\n"
-"      -l <ip[:port]> : set Destination IP [Port]\n"
+"      -k <ip[:port]> : set Destination IP [Port]\n"
+"      -l <ip[:port]> : set Source      IP [Port]\n"
+"      -t ttl         : set Time To Live\n"
 "      -w <file>      : write packet to this pcap file\n"
 "\n"
 "  Source options:\n"
@@ -107,6 +108,7 @@ struct options
     int mode;
     int pktlen;
     int prgalen;
+    int ttl;
 
     unsigned short sport;
     unsigned short dport;
@@ -155,6 +157,8 @@ int getip( char *s, unsigned char *ip , unsigned short *port)
         s++;
     }
 
+    if(i != 4) return 1;
+
     if( ( s = strchr( s, ':' ) ) && i == 4 )
     {
         s++;
@@ -363,6 +367,18 @@ int set_sport(unsigned char* packet)
     return 0;
 }
 
+int set_ip_ttl(unsigned char* packet)
+{
+    unsigned char ttl;
+
+    if(packet == NULL) return 1;
+
+    ttl = opt.ttl;
+    memcpy(packet+40, &ttl, 1);
+
+    return 0;
+}
+
 int set_IVidx(unsigned char* packet)
 {
     if(packet == NULL) return 1;
@@ -443,7 +459,7 @@ int read_raw_packet(unsigned char* dest, char* srcfile, int length)
     if(length  <= 0   ) return 1;
     if(length  >= 2048) return 1;
 
-    f = fopen(srcfile, "r");
+    f = fopen(srcfile, "rb");
     if(f == NULL)
     {
         perror("fopen failed.");
@@ -495,6 +511,7 @@ int write_cap_packet(unsigned char* packet, int length)
     if( fwrite( &pfh, 1, n, f ) != (size_t) n )
     {
         fprintf( stderr, "failed: fwrite(pcap file header)\n" );
+        fclose( f );
         return( 1 );
     }
 
@@ -510,6 +527,7 @@ int write_cap_packet(unsigned char* packet, int length)
     if( fwrite( &pkh, 1, n, f ) != (size_t) n )
     {
         fprintf( stderr, "fwrite(packet header) failed\n" );
+        fclose( f );
         return( 1 );
     }
 
@@ -518,9 +536,12 @@ int write_cap_packet(unsigned char* packet, int length)
     if( fwrite( packet, 1, n, f ) != (size_t) n )
     {
         fprintf( stderr, "fwrite(packet data) failed\n");
+        fclose( f );
         return( 1 );
     }
-
+	
+    fclose( f );
+	
     return 0;
 }
 
@@ -532,7 +553,12 @@ int read_prga(unsigned char **dest, char *file)
     if(file == NULL) return( 1 );
     if(*dest == NULL) *dest = (unsigned char*) malloc(1501);
 
-    f = fopen(file, "r");
+    if( memcmp( file+(strlen(file)-4), ".xor", 4 ) != 0 )
+    {
+        printf("Is this really a PRGA file: %s?\n", file);
+    }
+
+    f = fopen(file, "rb");
 
     if(f == NULL)
     {
@@ -541,20 +567,26 @@ int read_prga(unsigned char **dest, char *file)
     }
 
     fseek(f, 0, SEEK_END);
-    size = ftell(f);
+    size = (int)ftell(f);
     rewind(f);
 
     if(size > 1500) size = 1500;
 
-    if( fread( (*dest), size, 1, f ) != 1 )
+    if( (int)fread( (*dest), size, 1, f ) != 1 )
     {
         fprintf( stderr, "fread failed\n" );
+        fclose( f );
         return( 1 );
     }
 
+    if( (*dest)[3] > 0x03 )
+    {
+        printf("Are you really sure that this is a valid keystream? Because the index is out of range (0-3): %02X\n", (*dest)[3] );
+    }
+
     opt.prgalen = size;
 
-    fclose(f);
+    fclose( f );
     return( 0 );
 }
 
@@ -594,6 +626,8 @@ int forge_udp()
 
     if( set_dip(h80211, 48)  != 0 ) return 1;
     if( set_sip(h80211, 44)  != 0 ) return 1;
+    if( opt.ttl != -1 ) 
+        if( set_ip_ttl(h80211) != 0 ) return 1;
 
     /* set udp length */
     h80211[57] = '\x09';
@@ -624,6 +658,8 @@ int forge_icmp()
 
     if( set_dip(h80211, 48)  != 0 ) return 1;
     if( set_sip(h80211, 44)  != 0 ) return 1;
+    if( opt.ttl != -1 ) 
+        if( set_ip_ttl(h80211) != 0 ) return 1;
 
     /* generate + set ip checksum */
     chksum = ip_chksum((unsigned short*)(h80211+32), 20);
@@ -656,7 +692,7 @@ int forge_custom()
 
 void print_usage(void)
 {
-    printf(usage, getVersion("Packetforge-ng", _MAJ, _MIN, _SUB_MIN, _BETA) );
+    printf(usage, getVersion("Packetforge-ng", _MAJ, _MIN, _SUB_MIN, _REVISION) );
 }
 
 int main(int argc, char* argv[])
@@ -681,6 +717,7 @@ int main(int argc, char* argv[])
     opt.mode    = -1;
     opt.pktlen  = -1;
     opt.prgalen = -1;
+    opt.ttl     = -1;
 
     opt.sport   = -1;
     opt.dport   = -1;
@@ -691,8 +728,6 @@ int main(int argc, char* argv[])
 
     while( 1 )
     {
-        option_index = 0;
-
         static struct option long_options[] = {
             {"arp",      0, 0, '0'},
             {"udp",      0, 0, '1'},
@@ -701,8 +736,10 @@ int main(int argc, char* argv[])
             {0,          0, 0,  0 }
         };
 
-        int option = getopt_long( argc, argv,
-                        "p:a:c:h:jok:l:j:r:y:0129:w:e",
+        int option;
+	option_index = 0;
+	option = getopt_long( argc, argv,
+                        "p:a:c:h:jok:l:j:r:y:0129:w:et:",
                         long_options, &option_index );
 
         if( option < 0 ) break;
@@ -724,6 +761,17 @@ int main(int argc, char* argv[])
                 opt.fctrl[1]=(arg&0xFF);
                 break;
 
+            case 't' :
+
+                sscanf( optarg, "%i", &arg );
+                if( arg < 0 || arg > 255 )
+                {
+                    printf( "Invalid time to live.\n" );
+                    return( 1 );
+                }
+                opt.ttl = arg;
+                break;
+
             case 'a' :
 
                 if( getmac( optarg, 1, opt.bssid ) != 0 )
@@ -925,6 +973,7 @@ int main(int argc, char* argv[])
         printf("Error writing pcap file %s.\n", opt.cap_out);
         return 1;
     }
+    else printf( "Wrote packet to: %s\n", opt.cap_out );
 
     return 0;
 }
